/*****************************************************************
PAD 271008: Creation des fonctions My_max et My_min pour remplacer
les fonctions _max et __min utilisees par Laurent sous windows car
elles sont incompatibles ? d'autres compilateurs aux normes ANSI.
******************************************************************/
#include "Maiden_struct.h"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <vector>

//Some versions of math.h do not define M_PI PAD 230519
#ifndef M_PI
#define M_PI		3.14159265358979323846
#endif

double My_max(double value1, double value2)  // test ok 271008
{ return ( (value1 > value2) ? value1 : value2); }

double My_min(double value1, double value2) // test ok 271008
{ return ( (value1 < value2) ? value1 : value2); }

double Eps_eq_d18O(double TK) // Majoube (1971) PAD 131108
//Majoube M. (1971) Fractionnement en oxygen et en deuterium entre l'eau et sa vapeur. Journal de Chimie et de Physique 68, 1423.
/* Isotopic fractionation factor at the equilibrium of oxygen
   isotopes liquid-vapor (Per Mil) */
//{return ( (1137./(pow(TK,2.)) -0.4156/TK -0.00207)*1000. );}
/* Horita and Wesolowski, 1994 in Gibson et al., 2008 */
{return (exp((-7.685+6.7123*1000./TK-1.6664*1000000./pow(TK,2.)+0.35041*1000000000./pow(TK,3.)) /1000.)-1.)*1000.;} //permil

double Eps_eq_d2H(double TK) // Majoube (1971) PAD 131108
/* Fractionation factor at the equilibrium of hydrogen
   isotopes liquid-vapor (Per Mil) */
//{ return ( (24844./(pow(TK,2.)) -76.248/TK +0.052612)*1000. );}
/* Horita and Wesolowski, 1994 in Gibson et al., 2008 */
{return (exp((1158.8*pow(TK,3.)/1000000000.-1620.1*pow(TK,2.)/1000000.+794.84*TK/1000.-161.04+2.9992*1000000000./pow(TK,3.)) /1000.)-1.)*1000.;}

double Anderson(double d18Oxw, double d18Ov, double TK_leaf, double RH_leaf, double ff, double Eps_bio_value, double Eps_Kin_value) //  (19) PAD 141108
/* Oxygen isotopes ratios in Tree Ring Cellulose*/ // Danis et al. (2012) formulation of the Craig-Gordon model
{
  //double Eps_Kin;
  //double Eps_bio; FG
  //Eps_Kin = 26.351;//FG To calibrate?
  //Eps_bio = 27.0; FG

  return (1000.*((1.-ff)*(Eps_eq_d18O(TK_leaf)/1000.+Eps_Kin_value/1000.*(1.-RH_leaf)
			  +RH_leaf*d18Ov/1000.+ (1.-RH_leaf)*d18Oxw/1000.)
		 +ff*d18Oxw/1000.
		 +Eps_bio_value/1000.) );
}

//-- Pression de vapeur saturante en Pa
double P_sat1(double temp_K)
{
  return (101.325*exp(13.3185*(1.-(373.15/temp_K))
		      -1.9760*pow((1.-(373.15/temp_K)),2.)
		      -0.6445*pow((1.-(373.15/temp_K)),3.)
		      -0.1299*pow((1.-(373.15/temp_K)),4.)
		      )*1000.);}

double P_sat2(double temp_C) // tC en degres Celsuis (see throught.cpp line 81)
{ // comparable ? P_sat1
  return (610.7 * exp(17.38 * temp_C / (239.0 + temp_C)));}

double d18Oresi_evap(double TairK, double TwatK, double RHair, double d18Owat, double d18Ovap, double n, double W, double E ) // PAD 270819
/*                    Kelvin        Kelvin           1/1         Per Mil         Per Mil       1/1  */
{
  double Eps_LV=(exp((-7.685+6.7123*1000./TwatK-1.6664*1000000./pow(TwatK,2.)+0.35041*1000000000./pow(TwatK,3.)) /1000.)-1.)*1000.; //eps^+ (permil)
  double alpha_eq_LV = Eps_LV/1000.+1.; //alpha^+ > 1
  //double n = 1.       ; /* for soil water and for canopy (Gat, 1996) */
  double Ck = 28.6    ; /* for oxygen-18 (permil) */
  double RHwat = RHair*P_sat1(TairK)/P_sat1(TwatK); // (1/1)
  double theta = (1.-RHwat)/(1.-RHair); // (1/1)
  double Eps_k = n*Ck*theta*(1.-RHair); // (permil)
  return ( d18Owat*W - E / (1.-RHair+Eps_k/1000.) * ( -Eps_LV/alpha_eq_LV-RHair*d18Ovap-Eps_k) ) / ( W - E + E / (1.-RHair+Eps_k/1000.) / alpha_eq_LV );}

double d2Hresi_evap(double TairK, double TwatK, double RHair, double d2Hwat, double d2Hvap, double n, double W, double E ) // PAD 270819
/*                    Kelvin        Kelvin           1/1         Per Mil         Per Mil       1/1  */
{
  double Eps_LV=(exp((1158.8*pow(TwatK,3.)/1000000000.-1620.1*pow(TwatK,2.)/1000000.+794.84*TwatK/1000.-161.04+2.9992*1000000000./pow(TwatK,3.)) /1000.)-1.)*1000.; //eps^+ (permil)
  double alpha_eq_LV = Eps_LV/1000.+1.; //alpha^+ > 1
  //double n = 1.       ; /* for soil water and for canopy (Gat, 1996) */
  double Ck = 25.0    ; /* for deuterium (permil) */
  double RHwat = RHair*P_sat1(TairK)/P_sat1(TwatK); // (1/1)
  double theta = (1.-RHwat)/(1.-RHair); // (1/1)
  double Eps_k = n*Ck*theta*(1.-RHair); // (permil)
  return ( d2Hwat*W - E / (1.-RHair+Eps_k/1000.) * ( -Eps_LV/alpha_eq_LV-RHair*d2Hvap-Eps_k) ) / ( W - E + E / (1.-RHair+Eps_k/1000.) / alpha_eq_LV );}

double d18Oresi_subl(double TairK, double TwatK, double RHair, double d18Owat, double d18Ovap, double n, double W, double E ) // PAD 270819
/*                    Kelvin        Kelvin           1/1         Per Mil         Per Mil       1/1  */
{
  double alpha_eq_SV = exp( 0.0831 - 49.192/TwatK + 8312.5/TwatK/TwatK ); //Ellehoj et al., 2013
  double Eps_SV = 1000. * (alpha_eq_SV - 1.);
  //double n = 1.       ; /* for soil water and for canopy (Gat, 1996) */
  double Ck = 28.6    ; /* for oxygen-18 (permil) */
  double RHwat = RHair*P_sat1(TairK)/P_sat1(TwatK); // (1/1)
  double theta = (1.-RHwat)/(1.-RHair); // (1/1)
  double Eps_k = n*Ck*theta*(1.-RHair); // (permil)
  return ( d18Owat*W - E / (1.-RHair+Eps_k/1000.) * ( -Eps_SV/alpha_eq_SV-RHair*d18Ovap-Eps_k) ) / ( W - E + E / (1.-RHair+Eps_k/1000.) / alpha_eq_SV );}

double d2Hresi_subl(double TairK, double TwatK, double RHair, double d2Hwat, double d2Hvap, double n, double W, double E ) // PAD 270819
/*                    Kelvin        Kelvin           1/1         Per Mil         Per Mil       1/1  */
{
  double alpha_eq_SV = exp( 0.2133 - 203.1/TwatK + 48888./TwatK/TwatK ); //Ellehoj et al., 2013
  double Eps_SV = 1000. * (alpha_eq_SV - 1.);
  //double n = 1.       ; /* for soil water and for canopy (Gat, 1996) */
  double Ck = 25.0    ; /* for deuterium (permil) */
  double RHwat = RHair*P_sat1(TairK)/P_sat1(TwatK); // (1/1)
  double theta = (1.-RHwat)/(1.-RHair); // (1/1)
  double Eps_k = n*Ck*theta*(1.-RHair); // (permil)
  return ( d2Hwat*W - E / (1.-RHair+Eps_k/1000.) * ( -Eps_SV/alpha_eq_SV-RHair*d2Hvap-Eps_k) ) / ( W - E + E / (1.-RHair+Eps_k/1000.) / alpha_eq_SV );}

double d18Oevap(double TairK, double TwatK, double RHair, double d18Owat, double d18Ovap, double n ) // PAD 211108
/*                    Kelvin        Kelvin           1/1         Per Mil         Per Mil       1/1  */
/* n = 1. for soil and canopy, n = 0.5 for open water bodies like snow (Gat, 1996) */
/* Isotopic composition of evapored water from d18Owat (Per Mil)  */
/* Assumptions : liquide temperature equals air temperature       */ //now only needed for canopy PAD 2308819
/* See PAD's handbook page 74 and Evaporation_iso.xls    */
/* Http://www.iaea.org/programmes/ripc/ih/volumes/volumes.htm (vol 3 chap 1) */
{ /*
  double Delta_diff = -14.3 ; // [PerMil] Araguas-Araguas et al., 2000
  double alpha_eq_VL = exp(-Eps_eq_d18O(TwatK)/1000.) ;
  double Eps_VL=(alpha_eq_VL-1.)*1000. ;
  double RHwat = RHair*P_sat1(TairK)/P_sat1(TwatK);
  return ( (Alpha_eq_VL*d18Owat/1000.-RHwat*d18Ovap/1000.+Eps_VL/1000.+(1.-RHwat)*Delta_diff/1000.)
  /( (1.-RHwat)*(1.-Delta_diff/1000.) ) * 1000. ); // [Per Mil] */

  /* Following Gibson et al., 2008 (Eq. 1)*/
  double Eps_LV=Eps_eq_d18O(TwatK)  ; //eps^+ (permil)
  double alpha_eq_LV = Eps_LV/1000.+1.; //alpha^+ > 1
  //double n = 1.       ; /* for soil water and for canopy (Gat, 1996) */
  double Ck = 28.6    ; /* for oxygen-18 (permil) */
  double RHwat = RHair*P_sat1(TairK)/P_sat1(TwatK); // (1/1)
  double theta= (1.-RHwat)/(1.-RHair) ; // (1/1)
  double Eps_k=n*Ck*theta*(1.-RHair); // (permil)
  return ( 1000. / (1.-RHair+Eps_k/1000.) *
	   ( (d18Owat/1000.-Eps_LV/1000.)/alpha_eq_LV-RHair*d18Ovap/1000.-Eps_k/1000.) );}
  //return ( 1000. / (1.-RHair+Eps_k/1000.) *
//	   ( (d18Owat/1000.-Eps_LV/1000.)/alpha_eq_LV-RHair*d18Ovap/1000.-Eps_k/1000.) );}

double d2Hevap(double TairK, double TwatK, double RHair, double d2Hwat, double d2Hvap, double n ) // PAD 211108
/*                   Kelvin        Kelvin           1/1        Per Mil        Per Mil       1/1  */
/* n = 1. for soil and canopy, n = 0.5 for open water bodies like snow (Gat, 1996) */
/* Isotopic composition of evapored water from d18Owat (Per Mil) */
{ /*
  double Delta_diff = -12.5 ; // [PerMil] Araguas-Araguas et al., 2000
  double alpha_eq_VL = exp(-Eps_eq_d2H(TwatK)/1000.);
  double Eps_VL = (alpha_eq_VL-1.)*1000. ;
  double RHwat = RHair*P_sat1(TairK)/P_sat1(TwatK);
  return ( (alpha_eq_VL*d2Hwat/1000.-RHwat*d2Hvap/1000.+Eps_VL/1000.+(1.-RHwat)*Delta_diff/1000.)
  /( (1-RHwat)*(1.-Delta_diff/1000.) ) * 1000. ); // [Per Mil] */

  /* Following Gibson et al., 2008 (Eq. 1)*/
  double Eps_LV=Eps_eq_d2H(TwatK)  ; //eps^+
  double alpha_eq_LV = Eps_LV/1000.+1.;
  //double n = 1.       ; /* for soil water and for canopy (Gat, 1996) */
  double Ck = 25.0    ; /* for deuterium */
  double RHwat = RHair*P_sat1(TairK)/P_sat1(TwatK);
  double theta= (1.-RHwat)/(1.-RHair) ;
  double Eps_k=n*Ck*theta*(1.-RHair);
  return ( 1000. / (1.-RHair+Eps_k/1000.) *
	   ( (d2Hwat/1000.-Eps_LV/1000.)/alpha_eq_LV-RHair*d2Hvap/1000.-Eps_k/1000.) );}

double d18Osubl(double TairK, double TwatK, double RHair, double d18Owat, double d18Ovap, double n ) // PAD 2308819
/*                    Kelvin        Kelvin           1/1         Per Mil         Per Mil       1/1  */
/* n = 1. for soil and canopy, n = 0.5 for open water bodies like snow (Gat, 1996) */
/* Isotopic composition of sublimated snow from d18Owat (Per Mil)  */
{
  /* Following Gibson et al., 2008 (Eq. 1)*/
  double alpha_eq_SV = exp( 0.0831 - 49.192/TwatK + 8312.5/TwatK/TwatK ); //Ellehoj et al., 2013
  double Eps_SV = 1000. * (alpha_eq_SV - 1.);
  //double n = 0.5       ; /* for open water bodies */
  double Ck = 28.6    ; /* for oxygen-18 */
  double RHwat = RHair*P_sat1(TairK)/P_sat1(TwatK);
  double theta= (1.-RHwat)/(1.-RHair) ;
  double Eps_k=n*Ck*theta*(1.-RHair);
  return ( 1000. / (1.-RHair+Eps_k/1000.) *
	   ( (d18Owat/1000.-Eps_SV/1000.)/alpha_eq_SV-RHair*d18Ovap/1000.-Eps_k/1000.) );}

double d2Hsubl(double TairK, double TwatK, double RHair, double d2Hwat, double d2Hvap, double n ) // PAD 2308819
/*                   Kelvin        Kelvin           1/1        Per Mil        Per Mil       1/1  */
/* n = 1. for soil and canopy, n = 0.5 for open water bodies like snow (Gat, 1996) */
/* Isotopic composition of evapored water from d2Hwat (Per Mil) */
{
  /* Following Gibson et al., 2008 (Eq. 1)*/
  double alpha_eq_SV = exp( 0.2133 - 203.1/TwatK + 48888./TwatK/TwatK ); //Ellehoj et al., 2013
  double Eps_SV = 1000. * (alpha_eq_SV - 1.);
  //double n = 0.5.       ; /* for open water bodies */
  double Ck = 25.0    ; /* for deuterium */
  double RHwat = RHair*P_sat1(TairK)/P_sat1(TwatK);
  double theta= (1.-RHwat)/(1.-RHair) ;
  double Eps_k=n*Ck*theta*(1.-RHair);
  return ( 1000. / (1.-RHair+Eps_k/1000.) *
	   ( (d2Hwat/1000.-Eps_SV/1000.)/alpha_eq_SV-RHair*d2Hvap/1000.-Eps_k/1000.) );}

double AirVolumicMass(double TairK, double RHair, double Patmo) // PAD 21208
/*                          Kelvin           1/1           Pa
			    Volumic mass of air in Kg/m3 */
{
  double Rs = 287.05; // Specific constante of dry air [J/Kg/K]
  double Rv = 461;    // Specific constante of wet air [J/Kg/K]
  double Rh = Rs/(1.-(RHair * P_sat1(TairK) / Patmo)*(1-Rs/Rv));
  return ( Patmo / Rh / TairK );}


double density_h2o(double TairC, double Patmo) // AL 13032020
/*     calculate density of water at Patm (kg/m3)
 density_h2o <- function( tc, patm ){
   #-----------------------------------------------------------------------
   # Input:    - float, air temperature (tc), degrees C
   #           - float, atmospheric pressure (p), Pa
   # Output:   float, density of water, kg/m^3
   # Features: Calculates density of water at a given temperature and
   #           pressure using the Tumlirz Equation
   # Ref:      F.H. Fisher and O.E Dial, Jr. (1975) Equation of state of
   #           pure water and sea water, Tech. Rept., Marine Physical
   #           Laboratory, San Diego, CA.
   #-----------------------------------------------------------------------
 */
{
    double tc = TairC; // Tair in degrees C
    
   // Calculate lambda, (bar cm^3)/g:
   double my_lambda = 1788.316 + 21.55053*tc -0.4695911*tc*tc + (3.096363e-3)*tc*tc*tc-(7.341182e-6)*tc*tc*tc*tc;
   
   // Calculate po, bar
   double po = 5918.499 + 58.05267*tc -1.1253317*tc*tc + (6.6123869e-3)*tc*tc*tc -(1.4661625e-5)*tc*tc*tc*tc;
   
   // Calculate vinf, cm^3/g
   double vinf = 0.6980547 -(7.435626e-4)*tc +(3.704258e-5)*tc*tc -(6.315724e-7)*tc*tc*tc + (9.829576e-9)*tc*tc*tc*tc -(1.197269e-10)*tc*tc*tc*tc*tc +(1.005461e-12)*tc*tc*tc*tc*tc*tc  -(5.437898e-15)*tc*tc*tc*tc*tc*tc*tc +(1.69946e-17)*tc*tc*tc*tc*tc*tc*tc*tc -(2.295063e-20)*tc*tc*tc*tc*tc*tc*tc*tc*tc;
   
   // Convert pressure to bars (1 bar <- 100000 Pa)
   double pbar = (1e-5)*Patmo;
   
   // Calculate the specific volume (cm^3 g^-1):
    double v = vinf + my_lambda/(po + pbar);
   
   // Convert to density (g cm^-3) -> 1000 g/kg; 1000000 cm^3/m^3 -> kg/m^3:
    double rho = (1e3/v);
   
   return(rho);
 }


double viscosity_h2o(double TairC, double Patmo) // AL 13032020
/*  calculate viscosity of water (mu) in Pa/s
viscosity_h2o <- function( tc, patm ) {
  #-----------------------------------------------------------------------
  # Input:    - float, ambient temperature (tc), degrees C
  #           - float, ambient pressure (p), Pa
  # Return:   float, viscosity of water (mu), Pa s
  # Features: Calculates viscosity of water at a given temperature and
  #           pressure.
  # Depends:  density_h2o
  # Ref:      Huber, M. L., R. A. Perkins, A. Laesecke, D. G. Friend, J. V.
  #           Sengers, M. J. Assael, ..., K. Miyagawa (2009) New
  #           international formulation for the viscosity of H2O, J. Phys.
  #           Chem. Ref. Data, Vol. 38(2), pp. 101-125.
  #-----------------------------------------------------------------------
 */
 {
     double tc = TairC; // Tair in degrees C
     
  // Define reference temperature, density, and pressure values:
     double tk_ast  = 647.096;    // Kelvin
     double rho_ast = 322.0;      // kg/m^3
     double mu_ast  = 1e-6;      // Pa s
  
  // Get the density of water, kg/m^3
     double rho = density_h2o(tc, Patmo);
  
  // Calculate dimensionless parameters:
     double tbar = (tc + 273.15)/tk_ast;
     double tbarx = pow(tbar,0.5);
     double tbar2 = pow(tbar,2);
     double tbar3 = pow(tbar,3);
     double rbar = rho/rho_ast;
  
  // Calculate mu0 (Eq. 11 & Table 2, Huber et al., 2009):
     double mu0;
     
     mu0 = 1.67752 + 2.20462/tbar + 0.6366564/tbar2 - 0.241605/tbar3;
     mu0 = 1e2*tbarx/mu0;
  
  // Create Table 3, Huber et al. (2009):
     double h_array[7][6] = {
        {0.520094, 0.0850895, -1.08374, -0.289555, 0.0, 0.0} ,   /*  initializers for row indexed by 0 */
        {0.222531, 0.999115, 1.88797, 1.26613, 0.0, 0.120573} ,   /*  initializers for row indexed by 1 */
        {-0.281378, -0.906851, -0.772479, -0.489837, -0.257040, 0.0} ,  /*  initializers for row indexed by 2 */
         {0.161913,  0.257399, 0.0, 0.0, 0.0, 0.0} ,   /*  initializers for row indexed by 3 */
         {-0.0325372, 0.0, 0.0, 0.0698452, 0.0, 0.0} ,   /*  initializers for row indexed by 4 */
         {0.0, 0.0, 0.0, 0.0, 0.00872102, 0.0} ,  /*  initializers for row indexed by 5 */
         {0.0, 0.0, 0.0, -0.00435673, 0.0, -0.000593264}  /*  initializers for row indexed by 6 */
     };

  
  // Calculate mu1 (Eq. 12 & Table 3, Huber et al., 2009):
     double mu1 = 0.0;
     double ctbar = (1.0/tbar) - 1.0;
  // print(paste("ctbar",ctbar))
  // for i in xrange(6):
     double coef1,coef2;
     
     int i;
 //for (i in 1:6){
     for ( i = 0; i < 6; i++ ){
         coef1 = pow(ctbar,i);
         // print(paste("i, coef1", i, coef1))
         coef2 = 0.0;
      
         int j;
         for ( j = 0; j < 7; j++ ){
             coef2 = coef2 + h_array[j][i] * pow((rbar - 1.0),j);
     }

      mu1 = mu1 + coef1 * coef2;
  }

     mu1 = exp( rbar * mu1 );
  // print(paste("mu1",mu1))
  
 // Calculate mu_bar (Eq. 2, Huber et al., 2009)
  //   assumes mu2 = 1
   double  mu_bar = mu0 * mu1;
  
  // Calculate mu (Eq. 1, Huber et al., 2009)
    double mu = mu_bar * mu_ast;    // Pa s
  
  return( mu );
}

double Lavergne_isoC3(double CCa_CO2, double CCi_CO2, double TairC, double Patmo)
// Discrimination model based on Farquhar et al. (1982)
{
  // Variables IN
  //double CCa_CO2 ;  // Concentration of CO2 in the atmosphere [ppmV]
  //double CCi_CO2  ; // ConCentration of CO2 in the stomata [ppmV]
  //double TairC ;    // Air temperature [C]
  // double Patmo;    // Atmospheric pressure (Pa)
  //double Rd  ;      // Rate of mitochondrial respiration
  // double A;       //

  // Variables OUT
  double Cratio  ;      //  CCi_CO2/CCa_CO2 [1]
  double as,b,f; //ab,e,am; // isotopic fractionation factors
  double gamma;    // CO2 compensation point (ppmV), at 25C
  double delC3 ;        // Delta d13C

  //  define isotopic fractionation parameters

  //ab=2.9  // isotopic fractionations associated with diffusion through the laminar boundary layer
  as = 4.4; // isotopic fractionations associated with diffusion in air
  //am = 1.8; // isotopic fractionations associated with diffusion in water
  b=28; //  isotopic fractionation due to carboxylation by Rubisco; 28 +/- 2 permil
  //e=1.0 ; // isotopic fractionation during respiration;
  f=12;    // isotopic fractionation during photorespiration; 12+/- 4 permil
  //f=(-8.0+(350.-CCa_CO2)/7.);    // From C. Hatte & Guiot, 2005
    
  Cratio=CCi_CO2/CCa_CO2; // [ppmV/ppmV]
    
 // Calculation CO2 compensation point
    // Define constants
    //gs25 = 42.75;  // ppmV, value at 25C based on Bernacchi et al. (2001)
    //dha  = 37830;    // J/mol
    //kR   = 8.3145;   // J/mol/K
    //kPo = 101325   // atmospheric pressure at sea level, Pa (Allen, 1973)
    
    gamma = 42.75 * exp( 37830 * ( TairC - 25.0 ) / ( 298.15 * 8.3145 * ( TairC + 273.15 ) ) );
    
  // calculate the discrimination against 13C
    
    // discrimination model as in Busch et al. (2020) Nature Plants
 // delC3 =  ab*(1-CCs_CO2/CCa_CO2)+as*(CCs_CO2/CCa_CO2-Cratio)+am*(Cratio-CCc_CO2/CCa_CO2)+b*CCc_CO2/CCa_CO2-Rd/A*e*CCc_CO2/CCa_CO2-f*gamma/CCa_CO2;
    
    // discrimination model as in Seibt et al. (2008) Oecologia, neglecting fractionation due to respiration 
    //delC3 = as + (b-as)*Cratio - (b-am)*A/(gm*CCa_CO2) - f*gamma/CCa_CO2;
      
    // discrimination model simplified: assumptions are CCs_CO2 = CCa_CO2 and CCi_CO2 == CCc_CO2, and mesophyll and respiration fractionations negligible
    delC3 = as + (b-as)*Cratio - f*gamma/CCa_CO2;
    
  return (delC3);
}


double Hatte_isoC3(double CCa_CO2, double CCi_CO2, double TairC)
//,double Rd) // PAD 220409
// This part calculates fractionation for C3 photosynthesis.
// Christine Hatte (Christine.Hatte@lsce.ipsl.fr)
 /*
voici la routine qui permet de calculer le fractionnement (Delta13C) dans la feuille simule par BIOME4.
j'ai mis pas mal de choses en commentaire sur les choses que j'ai chang?es dans la routine originale,
donc des indications sur les gammes possibles de variations des coefficients.

* Attention, c'est bien le Delta d13C entre la feuille et l'atmopshere qui est simule.
il faut donc "ajouter" le d13C de l'atmosphere.
* en ce qui concerne le coeff b et vu qu'il pourrait bien etre a l'origine de mauvais
resultats dans ORCHIDEE dans les zones temperees. il est peut-etre preferable de revenir
a un coefficient constant (27.5) avec une large gamme de variation (au moins +/- 2.5 perMil) que
de conserver la formulation  b=27.5 + (25-T)/2
* ce qui est simule c'est le Delta d13C du bulk et non celui de la cellulose.
J'ai mis en commentaire des valeurs de fractionnement entre bulk et cellulose sorties
de la literature. pour le chene, un facteur de -0.5 perMil semble le plus adapte.
pour le pin, faudra plutot tirer vers 0.7 perMil.
je ne suis pas tres au fait de la biblio sur le fractt entre bulk et cellulose,
mais je me renseigne pour au moins avoir une gamme de variation possible.
*/
{
  // Variables IN
  //double CCa_CO2 ;  // Concentration of CO2 in the atmosphere [ppmV]
  //double CCi_CO2  ; // ConCentration of CO2 in the stomata [ppmV]
  //double TairC ;    // Air temperature [C]
  //double Rd  ;      // ?? [??] FG

  // Variables OUT
  double Cratio  ;      //  CCi_CO2/CCa_CO2 [1]
  //double DeltaA ; // [?] FG
  double a,es,a1,b,e,f,gamma; //,Catm;,k FG
  double leaftemp  ;    // Leaf temperature [C ??]
  double q,r,s,t ;      // Variables for the calculation of the Delta d13C ?? [PerMil ??]
  double delC3 ;        // Delta d13C

  ///FG SEVERAL PARAMETERS ARE NOT AS IN PAD 2012. VALUES ARE CHANGED. I HAVE TO CHECK THEM ONCE I WILL WORK ON D13C.

  //  define fractionation parameters
  //CC_CO2=Ca*1.e+6; // ConCentration from [ppmV] to [1/1] ??
  a = 4.4; // Coefficient pour la diffusion depuis l'atmosphere jusqu'au site de carboxylation dans l'air et l'eau (normalt il devrait y avoir deux parties: une pour l'air et l'autre pour l'eau mais c'est simplifi? [PerMil] CH 060509
  es= 1.1; // ?? [??]
  a1= 0.; // FG a1 seems par->Dc_p, which is considered a posteriori in the new version of TRC_isotopes.cpp
  // a1= 0.7; // valeurs de fractionnement entre bulk ?? [PerMil ??] // FG

  b=27.5; // coefficient pour la fixation photosynthetique (ca correspond a l'activite de la Rubisco) [PerMil] CH 060509
  // e= 0.0
  // f= 8.0			! From J. Kaplan (initial version)

  //b=27.5+(25.-TairC)/2.;
  e=1.0 ; // essayer egale a 0	terme de la respiration // This is e = 0.1 in PAD 2012
  f=(-8.0+(350.-CCa_CO2)/7.);	// From C. Hatte & Guiot, 2005


  //if (Rd <= 0) Rd=0.01; FG
  //leaftemp = 1.05*(TairC+2.5); //
  leaftemp = 21.4; //
  gamma = 1.54*leaftemp;
  //Rd = Rd/(86400.0*12.0);
  //Catm = CCa_CO2 en ppmV //Ca*1.0e6;
  //k = Rd/11.0   ;         //From Farquhar et al. 1982 p. 126

  Cratio=CCi_CO2/CCa_CO2; // [ppmV/ppmV]
  // printf("CC CO2 : %8.2lf %8.2lf\n",CCa_CO2,CCi_CO2);
  // printf("CCi/CCa CO2 : %8.2lf\n",Cratio);
  // calculate the fractionation
  q = a*(1.-Cratio+0.025);
  r = 0.075*(es+a1);
  s = b*(Cratio-0.1);
  //t = (e*Rd/k+f*gamma)/CCa_CO2 ; //atm   ;
  t = (e*11.+f*gamma)/CCa_CO2 ; //Catm doit etre en 10E-6  ;
  /* Rd/k = 11 dans tous les cas. Donc pas besoin de connaitre Rd seul. Empiriquement ils ne peuvent pas acceder a Rd seulement. Je n'y ai pas touche. C'est un des parametres qui correspond a la diurnal (ou dark) respiration (ou photorespiration). C'est une valeur fixe (sans unit?) qui ne devrait surement ne pas l'etre. Ca vaudrait peut-etre bien la peine, dans un deuxieme temps d'optimisation, de lui laisser un degre de liberte dans une gamme de ?2. CH 060509 */

  // t=0.0		! initially put to zero in the original version

  delC3 = q+r+s-t;
  return (delC3);
}

void spline(double *x, double *rwp, int n, int nssy)
{
    //meteo->T_previous_Y[y-year0]
    //int taille = sizeof(meteo->T_previous_Y[0])/sizeof(double);
    int i= 0;
    std::vector<double> a1(n+2, 0), a2(n+2, 0), a3(n+2, 0), a4(n+2, 0), arg(n), p(n);
    std::vector<int> ssy(n, nssy);

    for (i=0; i<n ; i++)
    {
        arg[i] = cos(2*M_PI/ssy[i]);
        p[i] = (2*pow(arg[i]-1, 2))/(arg[i]+2);

        if(i>=4)
        {
            a1[i]=1;
        }
        if(i>=3)
        {
            a2[i]=-4+p[i-2];
        }
        if(i>=2)
        {
            a3[i]=6+p[i-2]*4;
            a4[i]=x[i-2]+x[i]-2*x[i-1];
        }
    }

    for (i=2; i<n ; i++)
    {
        a1[i] = a1[i] * a3[i - 2];
        a2[i] = (a2[i] - a1[i] * a2[i - 1]) * a3[i - 1];
        a3[i] = 1/sqrt(a3[i] - pow(a1[i],2) - pow(a2[i], 2));
        a4[i] = (a4[i] - a4[i - 2] * a1[i] - a4[i - 1] * a2[i]) * a3[i];
    }

    for (i=n-1; i>=2; i--)
    {
        a4[i] = (a4[i] - a4[i + 1] * a2[i + 1] - a4[i + 2] * a1[i + 2]) * a3[i];
    }

    for(i=0; i<n ; i++)
    {
        rwp[i] = x[i]-(a4[i]-2*a4[i+1]+a4[i+2]);
    }
}


//Calculates Mean and returns the result
double Mean (double *My_array, int My_size ){

	double sum = 0;
	int i;

	for  ( i = 0; i < My_size; i++ ){

		sum += My_array[i];

	}

	return ( sum / My_size );

}

//Calculates the Standard Deviation of a Sample and returns the result
//It is overloaded to call Mean() when not given Mean
double Samp_Std_Dev ( double *My_array, int My_size, double mean ){

	double sum = 0;
	int i;

	for ( i = 0; i < My_size; i++ ){

		sum += pow( ( mean - My_array[i] ), 2);

	}

	return sqrt( ( sum / ( My_size - 1) ) );

}

//Calculates the T Score of a value, or array of values, then returns the result(s)
//It is overloaded to call Mean(), or Samp_Std_Dev() depending on what values it is called with
//Due to the T Score being values with a sample mean not population it calls Samp_Std_Dev() instead of Pop_Std_Dev()
//***Note***: Since mean and standard deviation are inputed for the first function and last they are identical to their Z_Score() companions
void T_Score ( double *My_array, int My_size ){

	int i;
	double mean = Mean( My_array, My_size ), Std = Samp_Std_Dev( My_array, My_size, mean);
	//double * Z_array = new double [My_size];

	for ( i = 0; i < My_size; i++ )
	{

		My_array[i] = ( ( My_array[i] - mean ) / Std );

	}

	//return Z_array;

}


